﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;

namespace SpaceShips
{
    public class Player
    {
        /// <summary>
        /// Pozice hráče
        /// </summary>
        public Vector2 position;
        /// <summary>
        /// Skóre
        /// </summary>
        public int score;
        /// <summary>
        /// Počet nábojů
        /// </summary>
        private int ammunition;
        /// <summary>
        /// Šířka a výška hráče
        /// </summary>
        private int widthSpr, heightSpr;

        /// <summary>
        /// Vytvořené kulky
        /// </summary>
        public List<Bullet> bullets;
        /// <summary>
        /// Kulka
        /// </summary>
        private Bullet bullet;

        /// <summary>
        /// Šířka a výška mapy
        /// </summary>
        private int widthMap, heightMap;

        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="position">Startovní pozice hráče</param>
        /// <param name="widthMap">Šířka herní mapy</param>
        /// <param name="heightMap">Výška herní mapy</param>
        /// <param name="widthSpr">Šířka hráče</param>
        /// <param name="heightSpr">Výška hráče</param>
        public Player(Vector2 position, int widthMap, int heightMap, int widthSpr, int heightSpr)
        {
            this.position = position;      // Startovní pozice hráče
            this.widthMap = widthMap;      // Inicializace proměných
            this.heightMap = heightMap;
            this.widthSpr = widthSpr;
            this.heightSpr = heightSpr;
            bullets = new List<Bullet>();  // Seznam vystřelených kulek
            score = 0;                     // Scóre hráče, defaultní hodnota
            ammunition = 50;               // Počet nábojů, defaultní množství
        }

        /// <summary>
        /// Vykreslení hráče a jeho střel
        /// </summary>
        /// <param name="spriteBatch">Třída pro práci se sprity</param>
        /// <param name="sprPlayerShip">Sprit hráčovi lodi</param>
        /// <param name="sprBullet">Sprit střely</param>
        public void DrawPlayerAndBullets(SpriteBatch spriteBatch, Texture2D sprPlayerShip, Texture2D sprBullet)
        {
            // Vykreslení hráče
            spriteBatch.Draw(sprPlayerShip, position, Color.White);

            // Vykreslení střely
            if (bullets.Count > 0)
            {
                for (int i = 0; i < bullets.Count; i++)
                {
                    bullets[i].DrawBullet(spriteBatch, 23, 34, sprBullet); 
                }
            }
        }

        /// <summary>
        /// Vykreslení informací o hráčovi
        /// </summary>
        /// <param name="spriteBatch">Třída pro práci se sprity</param>
        /// <param name="font">Font písma</param>
        public void DrawInfo(SpriteBatch spriteBatch, SpriteFont font)
        {
            spriteBatch.DrawString(font, "Ammunition: " + ammunition, new Vector2(0, (heightMap - 48)), Color.White);
            spriteBatch.DrawString(font, "Score: " + score, new Vector2(0, (heightMap - 15)), Color.White);
            spriteBatch.DrawString(font, "Press \"ESC\" for Menu!", new Vector2((widthMap - 237), (heightMap - 15)), Color.White);
        }

        /// <summary>
        /// Ovládání hráče
        /// </summary>
        /// <param name="key">Stav stisknuté klávesy</param>
        /// <param name="speed">Rychlost hráče</param>
        /// <param name="laserGun">Zvuk střely</param>
        public void ControlPlayer(KeyboardState key, KeyboardState lastKey, float speed, SoundEffect laserGun)
        {
            // Ovládání klávesy hráče
            if (key.IsKeyDown(Keys.D) && ((position.X + widthSpr) < widthMap))
                position.X += speed;
            else if (key.IsKeyDown(Keys.A) && (position.X > 0))
                position.X -= speed;
            else if (key.IsKeyDown(Keys.S) && ((position.Y + (heightSpr + 45)) < heightMap))
                position.Y += speed;
            else if (key.IsKeyDown(Keys.W) && (position.Y > 0))
                position.Y -= speed;

            // Střelba hráče a přiřazení nové kulky do seznamu kulek
            if (key.IsKeyDown(Keys.Space) && NoPressLastKey(key, lastKey))
            {
                if (ammunition > 0)
                {
                    laserGun.Play();
                    ammunition -= 1;
                    bullet = new Bullet(this.position, 7);
                    bullets.Add(bullet);
                }
            }
        }

        /// <summary>
        /// Ovládání ("Let") a kontrolování kulky
        /// <param name="widthSpr">Šířka kulky</param>
        /// <param name="heightSpr">Výška kulky</param>
        public void ControlBullets(int widthSpr, int heightSpr)
        {
            if (bullets.Count > 0)
            {
                for (int i = 0; i < bullets.Count; i++)
                {
                    bullets[i].ControlBullet(widthSpr, heightSpr);
                    // Vymaže kulku, která je mimo mapu
                    if (bullets[i].position.Y < 0)
                        bullets.RemoveAt(i);
                }
            }
        }

        /// <summary>
        /// Kontroluje jestli není klávesa stisknuta 2x za sebou 
        /// </summary>
        /// <param name="key">Stav aktuální stisknuté klávesy</param>
        /// <param name="lastKey">Stav minulé stisknuté klávesy</param>
        /// <returns>Je / Není stisknuta 2x</returns>
        private bool NoPressLastKey(KeyboardState key, KeyboardState lastKey)
        {
            return (key.IsKeyDown(Keys.Space) && lastKey.IsKeyUp(Keys.Space));
        }
    }
}
