using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;


namespace SpaceShips
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class ComponentItem : Microsoft.Xna.Framework.DrawableGameComponent
    {
        /// <summary>
        /// Hlavn komponenta
        /// </summary>
        private MainGame mainGame;
        /// <summary>
        /// Font poloek v menu
        /// </summary>
        private SpriteFont fontItem;
        /// <summary>
        /// Pozice
        /// </summary>
        private Vector2 position;
        /// <summary>
        /// Poloka menu
        /// </summary>
        private MenuItem menuItem;
        /// <summary>
        /// Seznam poloek
        /// </summary>
        private List<MenuItem> items;
        /// <summary>
        /// Aktuln vybran poloka
        /// </summary>
        private int actualItem;
        /// <summary>
        /// Barva vybran poloky
        /// </summary>
        private Color selectedItem;
        /// <summary>
        /// Barva nevybran poloky
        /// </summary>
        private Color unselectedItem;
        /// <summary>
        /// Stav stisknut klvesy
        /// </summary>
        private KeyboardState key;
        /// <summary>
        /// Stav posledn stisknut klvesy
        /// </summary>
        private KeyboardState lastKey;

        public ComponentItem(MainGame mainGame, Vector2 position, SpriteFont fontItem): base(mainGame)
        {
            this.mainGame = mainGame;      // Instance na hlavn komponentu
            this.position = position;      // Pozice poloky
            this.fontItem = fontItem;      // Font poloek v menu
            items = new List<MenuItem>();  // Seznam poloek menu
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            actualItem = 0;              // Defaultn hodnota
            selectedItem = Color.Green;
            unselectedItem = Color.Red;

            base.Initialize();
        }

        protected override void LoadContent()
        {
            base.LoadContent();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            lastKey = key;
            key = Keyboard.GetState();

            if (NoPressLastKey(Keys.W) || NoPressLastKey(Keys.Up))
            {
                actualItem--;
                if (actualItem < 0)
                    actualItem = items.Count - 1;
            }
            else if (NoPressLastKey(Keys.S) || NoPressLastKey(Keys.Down))
            {
                actualItem++;
                if (actualItem > items.Count - 1)
                    actualItem = 0;
            }

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            mainGame.spriteBatch.Begin();

            if (items.Count > 0)
            {
                for(int i = 0; i < items.Count; i++)
                {
                    if (i == actualItem)
                        mainGame.spriteBatch.DrawString(fontItem, items[i].NameItem(), items[i].ReturnPosition(), Color.Green);
                    else
                        mainGame.spriteBatch.DrawString(fontItem, items[i].NameItem(), items[i].ReturnPosition(), Color.Red);
                }
            }

            mainGame.spriteBatch.End();

            base.Draw(gameTime);
        }

        /// <summary>
        /// Pid poloku do seznamu poloek
        /// </summary>
        /// <param name="textName">Jmno poloky</param>
        public void AddItem(string textName)
        {
            if (items.Count == 0)
                items.Add(menuItem = new MenuItem(this.position, textName));
            else
                items.Add(menuItem = new MenuItem(new Vector2(this.position.X, this.position.Y + (items.Count * 50)), textName));
        }

        /// <summary>
        /// Kontroluje jestli nen klvesa stisknuta 2x za sebou 
        /// </summary>
        /// <param name="key">Stav aktuln stisknut klvesy</param>
        /// <param name="lastKey">Stav minul stisknut klvesy</param>
        /// <returns>Je / Nen stisknuta 2x</returns>
        private bool NoPressLastKey(Keys keys)
        {
            return (key.IsKeyDown(keys) && lastKey.IsKeyUp(keys));
        }

        /// <summary>
        /// Vrt zvolenou poloku v menu
        /// </summary>
        /// <returns>Index poloky</returns>
        public int IsSelectedItem()
        {
            return actualItem;
        }
    }
}
