using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;


namespace SpaceShips
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class ComponentMenu : Microsoft.Xna.Framework.DrawableGameComponent
    {
        /// <summary>
        /// Hlavn komponenta
        /// </summary>
        private MainGame mainGame;
        /// <summary>
        /// Pozad menu
        /// </summary>
        private Texture2D sprBgMenu;
        /// <summary>
        /// Komponenta poloky menu
        /// </summary>
        private ComponentItem componentItem;
        /// <summary>
        /// Stav stisknut klvesy
        /// </summary>
        private KeyboardState key;
        /// <summary>
        /// Font nadpisu
        /// </summary>
        private SpriteFont font;
        /// <summary>
        /// Font pro npis autora
        /// </summary>
        private SpriteFont fontAuthor;
        /// <summary>
        /// Font pro poloky v menu
        /// </summary>
        private SpriteFont fontItem;
        /// <summary>
        /// Hudba v pozad menu
        /// </summary>
        private Song menuSong;

        public ComponentMenu(MainGame mainGame): base(mainGame)
        {
            this.mainGame = mainGame;      // Instance na hlavn komponentu
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            sprBgMenu = mainGame.Content.Load<Texture2D>("Textures/spr_bgMenu");
            font = mainGame.Content.Load<SpriteFont>("Fonts/RavieFont");
            fontAuthor = mainGame.Content.Load<SpriteFont>("Fonts/RavieAuthorFont");
            fontItem = mainGame.Content.Load<SpriteFont>("Fonts/AdobeFontLarger");
            menuSong = mainGame.Content.Load<Song>("Musics/SpaceLoop");

            CreateMenuItems();

            SetMusic();

            base.LoadContent();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            key = Keyboard.GetState();

            if (key.IsKeyDown(Keys.Enter))
            {
                switch (componentItem.IsSelectedItem())
                {
                    case 0:
                        // Vypnut komponent Menu a poloky menu
                        this.Enabled = false;
                        this.Visible = false;
                        componentItem.Enabled = false;
                        componentItem.Visible = false;
                        MediaPlayer.Stop();   // Vypnut hudby
                        // Deklarace a inicializace komponenty se hrou. Pidn komponenty do komponent
                        ComponentRoom componentRoom = new ComponentRoom(mainGame);
                        RemoveComponent(componentRoom);
                        mainGame.Components.Add(componentRoom);
                        break;

                    case 1:
                        mainGame.Exit();
                        break;
                }
            }

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            mainGame.spriteBatch.Begin();

            mainGame.spriteBatch.Draw(sprBgMenu, new Vector2(0, 0), Color.White);
            mainGame.spriteBatch.DrawString(fontAuthor, "Author: Martin Novak", new Vector2(0, mainGame.HEIGHT - 14), Color.YellowGreen);
            mainGame.spriteBatch.DrawString(font, "Space ships", new Vector2(65, 50), Color.Brown);

            mainGame.spriteBatch.End();

            base.Draw(gameTime);
        }

        /// <summary>
        /// Metoda mae duplicitn komponenty
        /// </summary>
        /// <param name="obj">Komponenta ke smazn</param>
        private void RemoveComponent(object obj)
        {
            for (int i = 0; i < mainGame.Components.Count; i++)
            {
                if (mainGame.Components[i].GetType() == obj.GetType())
                {
                    mainGame.Components.RemoveAt(i);
                }
            }
        }

        /// <summary>
        /// Nastav a spust hudbu
        /// </summary>
        public void SetMusic()
        {
            MediaPlayer.IsRepeating = true;  // Hudba se bude opakovat
            MediaPlayer.Play(menuSong);      // Sputn hudby
        }

        /// <summary>
        /// Metoda vytvo poloky menu
        /// </summary>
        public void CreateMenuItems()
        {
            componentItem = new ComponentItem(mainGame, new Vector2(150, 225), fontItem);
            RemoveComponent(componentItem);
            componentItem.AddItem("New Game");
            componentItem.AddItem("Exit game");
            mainGame.Components.Add(componentItem);
        }
    }
}
