using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;


namespace SpaceShips
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class ComponentRoom : Microsoft.Xna.Framework.DrawableGameComponent
    {
        /// <summary>
        /// Hlavn komponenta
        /// </summary>
        private MainGame mainGame;
        /// <summary>
        /// Pozad levelu
        /// </summary>
        private Texture2D sprBackground;
        /// <summary>
        /// Pozad pro menu a informace pro hre
        /// </summary>
        private Texture2D sprBgInfo;
        /// <summary>
        /// Stav stisknut klvesy a stav stisknut pedel klvesy
        /// </summary>
        private KeyboardState key, lastKey;
        /// <summary>
        /// Zvukov efekty
        /// </summary>
        private SoundEffect explosion, laserGun;
        /// <summary>
        /// Font psma pro vykreslen zprvy o prohe
        /// </summary>
        private SpriteFont fontGO;
        /// <summary>
        /// Udruje stav hry
        /// </summary>
        private bool isGame;
        /// <summary>
        /// Vyhrl / nevyhrl hr
        /// </summary>
        private bool win;
        /// <summary>
        /// Poet neptelskch lod
        /// </summary>
        private int numberEnemyShips;

        #region Player
        /// <summary>
        /// Tda s hrem
        /// </summary>
        private Player player;
        /// <summary>
        /// Startovn pozice hre
        /// </summary>
        private Vector2 position;
        /// <summary>
        /// Sprit hrovi lodi
        /// </summary>
        private Texture2D sprPlayerShip;
        /// <summary>
        /// Sprit stely
        /// </summary>
        private Texture2D sprPlayerBullet;
        /// <summary>
        /// Rychlost hre
        /// </summary>
        private float speedPlayer;
        /// <summary>
        /// Font psma, pro informace o hri
        /// </summary>
        private SpriteFont font;
        /// <summary>
        /// Kolize kulky s neptelem
        /// </summary>
        private bool bulletIsCol;
        #endregion
        #region Enemy
        /// <summary>
        /// Tda s neptelem
        /// </summary>
        private Enemy enemy;
        /// <summary>
        /// Ryhlost neptele
        /// </summary>
        private float speedEnemy;
        /// <summary>
        /// Aktivn neptel
        /// </summary>
        private List<Enemy> enemies;
        /// <summary>
        /// Sprit neptele
        /// </summary>
        private Texture2D sprEnemyShip;
        /// <summary>
        /// as pi generovn neptel
        /// </summary>
        private float generateTime;
        /// <summary>
        /// Generovn nhodnch souadnic, pro pozici neptele
        /// </summary>
        private Random generateCoordinates;
        #endregion

        public ComponentRoom(MainGame mainGame): base(mainGame)
        {
            this.mainGame = mainGame;   // Instance na hlavn komponentu
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            isGame = true;           // Defaultn stav
            win = false;   
            numberEnemyShips = 0;

            #region Player
            position = new Vector2(((mainGame.WIDTH / 2) - 20), (mainGame.HEIGHT - 160));  // Startovn pozice hre
            speedPlayer = 4;  // Rychlost hre
            player = new Player(position, mainGame.WIDTH, mainGame.HEIGHT, 50, 100);  // Instance hre
            bulletIsCol = false;   // Defaultn hodnota
            #endregion
            #region Enemy
            enemies = new List<Enemy>();          // Aktivn neptel ve he
            speedEnemy = 3;                       // Rychlost letu neptel
            generateTime = 0.0f;                  // Defaultn stav
            generateCoordinates = new Random();   // Instance tdy "Random"
            #endregion

            base.Initialize();
        }

        protected override void LoadContent()
        {
            sprBackground = mainGame.Content.Load<Texture2D>("Textures/spr_background");
            sprBgInfo = mainGame.Content.Load<Texture2D>("Textures/spr_bgInfo");
            explosion = mainGame.Content.Load<SoundEffect>("Effects/Explosion");
            laserGun = mainGame.Content.Load<SoundEffect>("Effects/LaserGun");
            fontGO = mainGame.Content.Load<SpriteFont>("Fonts/RavieGameOverFont");

            #region Player
            sprPlayerShip = mainGame.Content.Load<Texture2D>("Textures/spr_playerShip");
            sprPlayerBullet = mainGame.Content.Load<Texture2D>("Textures/spr_playerBullet");
            font = mainGame.Content.Load<SpriteFont>("Fonts/AdobeFont");
            #endregion
            #region Enemy
            sprEnemyShip = mainGame.Content.Load<Texture2D>("Textures/spr_enemyShip");
            #endregion

            base.LoadContent();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            lastKey = key;                                                 // Pedel stisknut klvesa
            key = Keyboard.GetState();                                     // Aktuln stav stisknut klvesy
            generateTime += (float)gameTime.ElapsedGameTime.TotalSeconds;  // Pit do promnn sekundy od poslednho Updatu

            // Oputn hry a nvrat do hlavnho menu
            if (key.IsKeyDown(Keys.Escape))
            {
                for (int i = 0; i < mainGame.Components.Count; i++)
                {
                    if (mainGame.Components[i] is ComponentMenu)
                    {
                        ComponentMenu obj = (ComponentMenu)mainGame.Components[i];
                        // Zapnut komponenty menu
                        obj.Enabled = true;
                        obj.Visible = true;
                        obj.CreateMenuItems();
                        obj.SetMusic();
                        // Vypnut tto komponenty
                        this.Enabled = false;
                        this.Visible = false;
                    }
                }
            }

            if (isGame && !win)
            {
                // Ovldn hre a jeho kulky
                player.ControlPlayer(key, lastKey, speedPlayer, laserGun);
                player.ControlBullets(sprPlayerBullet.Width, sprPlayerBullet.Height);

                // Generovn neptel
                if (numberEnemyShips <= 45)
                {
                    if (generateTime >= 1.0f)
                    {
                        enemies.Add(enemy = new Enemy(new Vector2(generateCoordinates.Next((0 + sprEnemyShip.Width), (mainGame.WIDTH - sprEnemyShip.Width)), -sprEnemyShip.Height), speedEnemy));  // Vytvoen a pidn novho neptele do hry
                        generateTime = 0.0f;  // Vynulovn asu pro dal generovn
                        numberEnemyShips++;   // Piten vytvoenho neptele
                    }
                }
                if (numberEnemyShips >= 45 && enemies.Count == 0)
                {
                    win = true;
                }

                CollisionEnemyAndPlayerBullets();   // Kolize hrovi kulky s neptelem

                // Ovldn neptele
                if (enemies.Count > 0)
                {
                    for (int i = 0; i < enemies.Count; i++)
                    {
                        enemies[i].ControlEnemy(sprEnemyShip.Width, sprEnemyShip.Height, player);  // Let neptele
                        // Kontrolovn, jestli neptel nevyjel z mapy nebo jestli nekolidoval s hrem
                        if (enemies[i].IsBehindMap(mainGame.HEIGHT, sprBgInfo.Height) || enemies[i].enemyRec.Contains(new Point(((int)player.position.X + sprPlayerShip.Width), ((int)player.position.Y + sprPlayerShip.Height))))
                        {
                            if (enemies[i].enemyRec.Contains(new Point(((int)player.position.X + sprPlayerShip.Width), ((int)player.position.Y + sprPlayerShip.Height))))
                            {
                                explosion.Play();      // Zvukov efekt nrazu
                                isGame = false;        // Zastav a ukon hru
                            }

                            if (player.score > 0)
                                player.score -= 5;     // Odebrn hrova skre
                            enemies.RemoveAt(i);   // Odstrann neptele
                        }
                    }
                }
            }

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            mainGame.spriteBatch.Begin();

            // Vykreslen levelu
            mainGame.spriteBatch.Draw(sprBackground, new Vector2(0, 0), Color.White);

            #region Enemy
            if (enemies.Count > 0)
            {
                for (int i = 0; i < enemies.Count; i++)
                {
                    enemies[i].DrawEnemy(mainGame.spriteBatch, sprEnemyShip);
                }
            }
            #endregion

            // Vykreslen pozad na informace hre
            mainGame.spriteBatch.Draw(sprBgInfo, new Vector2(0, (mainGame.HEIGHT - 50)), Color.Black);

            #region Player
            // Vykreslen informac o hri
            player.DrawInfo(mainGame.spriteBatch, font);
            // Vykreslen hre a jeho stel
            player.DrawPlayerAndBullets(mainGame.spriteBatch, sprPlayerShip, sprPlayerBullet);
            #endregion

            // Vykresl hlku o prohe
            if (!isGame || (win && player.score == 0))
                mainGame.spriteBatch.DrawString(fontGO, "Game Over!", new Vector2(70, 225), Color.Red);
            if (win && player.score > 0)
                mainGame.spriteBatch.DrawString(fontGO, "You win!", new Vector2(110, 225), Color.Green);

            mainGame.spriteBatch.End();

            base.Draw(gameTime);
        }

        /// <summary>
        /// Hld kolize neptele s kulkou hre
        /// </summary>
        /// <param name="enemyRec">Ohranien neptele</param>
        /// <param name="bulletRec">Ohranien kulky</param>
        private void CollisionEnemyAndPlayerBullets()
        {
            if (player.bullets.Count > 0 && enemies.Count > 0)
            {
                for (int i = 0; i < player.bullets.Count; i++)
                {
                    for (int z = 0; z < enemies.Count; z++)
                    {
                        if (enemies[z].enemyRec.Contains(player.bullets[i].bulletRec))
                        {
                            enemies.RemoveAt(z);  // Smazn neptele
                            bulletIsCol = true;   // Povolen smazn kulky
                        }
                    }
                    if (bulletIsCol)
                    {
                        player.score += 5;            // Piten scre hrovi
                        player.bullets.RemoveAt(i);  // Smazn kulky
                        bulletIsCol = false;         // Zakzn smazn kulky
                    }
                }
            }
        }
    }
}
